﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.AutoCast
{
	public class AutoCast<IHolder> : thrts.AutoCastParameter<IHolder>
	{
		public AutoCast(string name, int interval, Func<IHolder, bool> castAction, bool defaultEnable = false)
		{
			this.DefaultAutoCastEnable = defaultEnable;
			this.Name = name;
			this.CastInterval = interval;
			this.CastAction = castAction;
		}
	}
	public class ExpFlagAutoCast<IHolder> : AutoCast<IHolder>
	{
		bool onlyEnemy = true ;
		public ExpFlagAutoCast(string name,bool onlyEnemy = true)
			: base(name, thrts.Def.FPS/2, null, true)
		{
			this.CastAction = SetExpFlag;
			this.onlyEnemy = onlyEnemy;
		}
		public bool SetExpFlag(IHolder holder)
		{
			/// search target
			if (holder as thrts.IUnit != null)
			{
				var u = holder as thrts.IUnit;
				var selfFaction = u.Faction;
				var world = selfFaction.World;
				foreach (var tu in world.GetObjectFromCircle(new Circle2D(u.ViewRange,new Vector2D(u.Pos.X,u.Pos.Y))).OfType<IUnit>())
				{
					if( !onlyEnemy || !world.IsSameTeam(selfFaction, tu.Faction) )
					{
						var ability = tu.AbilityManager.GetAbilityWithName(thrts.definedAbility.SendExpAbility.GetAbilityNameWithDist(u));
						if(null != ability)
							ability.ExtendLifetime(CastInterval,CastInterval*2);
						else
							tu.AbilityManager.AppendAbility(new thrts.definedAbility.SendExpAbility(u, world.IsMaxDamage));
					}
				}
			}
			return true;
		}
	}
	public class ExpFlagWithCollectTokenAutoCast<IHolder> : AutoCast<IHolder>
	{
		bool onlyEnemy;
		public ExpFlagWithCollectTokenAutoCast(string name,bool onlyEnemy = true)
			: base(name, thrts.Def.FPS/2, null, true)
		{
			this.CastAction = SetExpFlag;
			this.onlyEnemy = onlyEnemy;
		}
		public bool SetExpFlag(IHolder holder)
		{
			/// search target
			if (holder as thrts.IUnit != null)
			{
				var u = holder as thrts.IUnit;
				var selfFaction = u.Faction;
				var world = selfFaction.World;
				foreach (var tu in world.GetObjectFromCircle(new Circle2D(u.ViewRange,new Vector2D(u.Pos.X,u.Pos.Y))).OfType<IUnit>())
				{
					if( !u.IsSummon && (!onlyEnemy || !world.IsSameTeam(selfFaction, tu.Faction)) )
					{
						var ability = tu.AbilityManager.GetAbilityWithName(Ability.SendExpAndCollectTokenAbility.GetAbilityNameWithDist(u));
						if(null != ability)
							ability.ExtendLifetime(CastInterval,CastInterval*2);
						else
							tu.AbilityManager.AppendAbility(new Ability.SendExpAndCollectTokenAbility(u, world.IsMaxDamage));
					}
				}
			}
			return true;
		}
	}
	public class HealUniqueAutoCast<IHolder> : AutoCast<IHolder>
	{
		int counter = 0;
		const int healDelay = 5*Def.FPS;
		float healHpRatio = 0.02f;
		int lastHP = 0;
		/// <summary>
		/// Initializes a new instance of the <see cref="LotusCraft.AutoCast.HealUniqueAutoCast`1"/> class.
		/// </summary>
		/// <param name='name'>
		/// Name.
		/// </param>
		/// <param name='healHpRatio'>
		/// Heal hp ratio per second.
		/// </param>
		public HealUniqueAutoCast(string name,float healHpRatio = 0.04f)
			: base(name, Def.FPS/2, null, true)
		{
			this.CastAction = AutoHeal;
			counter = 0;
			this.healHpRatio = healHpRatio;

		}
		public bool AutoHeal(IHolder holder)
		{
			/// check self state
			if (holder as thrts.IUnit != null)
			{
				var u = holder as thrts.IUnit;
				var selfFaction = u.Faction;
				if(lastHP <= u.HP && u.ActualAction == UnitAction.Free)
					++counter;
				else
					counter = 0;

				if(counter*CastInterval > healDelay && u.HP != u.MaxHP)
					u.HP += (int)(u.MaxHP*healHpRatio/2);

				lastHP = u.HP;
			}
			return true;
		}
	}

	public class EnqueueProductionUnitAutoCast : AutoCast<IBuilding>
	{
		const int queueCanselWait = 15;
		public EnqueueProductionUnitAutoCast (string name, int makeTime, ButtonPosition x, ButtonPosition y, bool canParaExec = false, bool isDefaultEnable = false)
			: base(name, Def.FPS, null, isDefaultEnable)
		{
			this.CastAction = PushButton;
			this.ButtonPosX = x;
			this.ButtonPosY = y;
			this.IsButtonSetup = true;
		}
		public override void OnToggle (IBuilding building, bool flag)
		{
			if (building.UserValues.ContainsKey (BuildingParameter.WaitAutoProductionKey)) {
				building.UserValues.Remove(BuildingParameter.WaitAutoProductionKey);
			}

			return;
		}

		public bool PushButton (IBuilding building)
		{
			if (null == building)
				return false;

			var button = building.TryGetButtonParameter (this.ButtonPosX, this.ButtonPosY);
			if (null == button)
				return false;

			if (building.UserValues.ContainsKey (BuildingParameter.WaitAutoProductionKey)) {
				int count = (int)building.UserValues[BuildingParameter.WaitAutoProductionKey];
				if(count < queueCanselWait)
				{
					building.UserValues[BuildingParameter.WaitAutoProductionKey] = count + 1;
					return false;
				}
				else
				{
					building.UserValues.Remove(BuildingParameter.WaitAutoProductionKey);
				}
			}

			if(building.GetCountInProductionQueue(this.Name) >= 1)
				return false;

			var executableInfo = button.CanExecute (building.Faction, new []{building}, building);
			if (executableInfo != null && executableInfo.CanExecute) {
				switch (button.Kind) {
				case ButtonKind.CreateBuilding:
					button.CreateBuildingEvent (building, false);
					break;
				case ButtonKind.CreateUnit:
					button.CreateUnitEvent (building, false);
					break;
				case ButtonKind.Normal:
					button.NormalButtonEvent (building, false);
					break;
				}

			}
			return true;
		}

	}

	public class ButtonPushAutoCast<IHolder> : AutoCast<IHolder>
	{
		bool canParallelExec;
		public ButtonPushAutoCast(string name, int interval, ButtonPosition x, ButtonPosition y, bool canParaExec = false, bool isDefaultEnable = false)
			: base(name, interval, null, isDefaultEnable)
		{
			this.CastAction = PushButton;
			this.ButtonPosX = x;
			this.ButtonPosY = y;
			this.canParallelExec = canParaExec;
			this.IsButtonSetup = true;

		}
		public bool PushButton (IHolder holder)
		{
			var u = holder as IUnit;
			var b = holder as IBuilding;
			/// check self state
			if (u != null) {
				var button = u.TryGetButtonParameter(this.ButtonPosX,this.ButtonPosY);
				if(button != null)
				{
					var executableInfo = button.CanExecute(u.Faction,new []{u},u);
					if( executableInfo != null && executableInfo.CanExecute)
					{
						switch(button.Kind)
						{
						case ButtonKind.CreateBuilding:
							button.CreateBuildingEvent(u, false);
							break;
						case ButtonKind.CreateUnit:
							button.CreateUnitEvent(u, false);
							break;
						case ButtonKind.Normal:
							button.NormalButtonEvent(u, false);
							break;
						}
					}
				}
			} else if (b != null) {
				var button = b.TryGetButtonParameter(this.ButtonPosX,this.ButtonPosY);
				if(button != null)
				{
					var executableInfo = button.CanExecute(b.Faction,new []{b},b);
					if( executableInfo != null && executableInfo.CanExecute)
					{
						switch(button.Kind)
						{
						case ButtonKind.CreateBuilding:
							button.CreateBuildingEvent(b, false);
							break;
						case ButtonKind.CreateUnit:
							button.CreateUnitEvent(b, false);
							break;
						case ButtonKind.Normal:
							button.NormalButtonEvent(b, false);
							break;
						}
					}
				}
			}
			return true;
		}
		public void onToggle (IHolder holder, IAutoCast<IHolder> autocast)
		{
			var u = holder as IUnit;
			var b = holder as IBuilding;
			/// check self state
			if (u != null) {

			} else {

			}
		}
	}
}
